
function [T,Y]=Linear_ODE(t0,tmax,alfa,beta,N,m,p,q,r);
%________________________________________________________________________
%
% Funzione Linear_ODE 
% ____________________
%
% Chiamata:   [T,Y]=Linear_ODE(t0,tmax,alfa,beta,N,m,p,q,r);
%
% Commento:
%
% La funzione 'Linear_ODE' utilizza il comando Matlab "\" 
% per calcolare la soluzione del sistema lineare ottenuto 
% approssimando con le formule alle differenze finite y' e 
% y'' in un problema lineare ai limiti del 2 ordine della forma:
%
% y''(t)=p(t)*y'+q(t)*y+r(t)     con t in (t0,tmax)
% a0*y(t0)-a1*y'(t0)=alfa
% b0*y(tmax)+b1*y'(tmax)=beta
%
%
% dove a0, b0, a1 e b1 sono tali che: |a0|+|a1|>0
%                                     |b0|+|b1|>0 .
%
%________________________________________________________________________
% 
%
% Parametri di input della function:
%
% t0,tmax    estremi dell'intervallo [t0,tmax] di integrazione;         
% alfa, beta condizioni ai limiti;
% N          numero di sottointervalli in cui e' stato suddiviso [t0,tmax]; 
% m          matrice di dimensione 2 x 2 contenente i coefficienti delle 
%            combinazioni di y e y' ai limiti; si  indicato:
%
%            |a0  a1|    |m(1,1) m(1,2) |
%            |b0  b1| =  |m(2,1) m(2,2) | .
%           
%            Quando a1=0 (b1=0) si deve porre a0=1 (b0=1);  
% p, q, r    stringhe contenenti le espressioni delle funzioni p(t), q(t), 
%            r(t) nella variabile t.
%
% Parametri di output della function:
%
% T          vettore colonna di dimensione N+1 contenente i nodi t0,t1,...,tn;
% Y          vettore colonna di dimensione N+1 contenente l'approssimazione
%            della soluzione, ottenuta dal metodo, calcolata nei nodi ti,
%            i=0,...,n.
%_________________________________________________________________________
%
% Esempio  di utilizzo della function  applicata al seguente 
% problema ai limiti:
%
% t^2*y'' + t*y' - y = t^4    t in (1,2)
% y(1)=1; 		  
% y(2)=2; 		  
% 
% con soluzione vera:
% y(t)=28/45*t.^(-1)+14/45*t+1/15*t.^4;
%
%________________________________________________________________________
%
% clear all; clc
% t0=1;tmax=2;
% alfa=1;
% beta=2;
% p='-1./t';
% q='1./(t.^2)';
% r='t.^2';
% m=[1 0; 1 0];
% N=20;
% 
% %Implementazione del metodo
% 
% [T,Y]=Linear_ODE(t0,tmax,alfa,beta,N,m,p,q,r);
% 
%________________________________________________________________________
%

h=(tmax-t0)/N;
t=(t0:h:tmax)';      
t(end)=tmax;         

p=eval(p).*ones(size(t));
q=eval(q).*ones(size(t));
r=eval(r).*ones(size(t));
%
% 1 CASO: y(t0)=alfa, y(tmax)=beta
%
if m(1,2)==0 & m(2,2)==0
    A=diag((1+h/2*p(3:N)),-1)+diag(-(2+h^2*q(2:N)))+diag((1-h/2*p(2:N-1)),1);
    b=r(2:N)*h^2;
    b(1)=b(1)-alfa*(1+p(2)*h/2);
    b(N-1)=b(N-1)-beta*(1-p(N)*h/2);
    
end
%
% 2 CASO: y(t0)=alfa,  b0*y(tmax)+b1*y'(tmax)=beta
%
if m(1,2)==0 & m(2,2)~=0
    A=diag((1+h/2*p(3:N+1)),-1)+diag((-2-h^2*q(2:N+1)))+diag((1-h/2*p(2:N)),1);
    A(N,N-1)=2*m(2,2);
    A(N,N)=m(2,2)*A(N,N)-2*h*m(2,1)*(1-p(N+1)*h/2);

    b=r(2:N+1)*h^2;
    b(1)=b(1)-alfa*(1+p(2)*h/2);
    b(N)=m(2,2)*b(N)-2*h*beta*(1-p(N+1)*h/2);
end
%
% 3 CASO:  a0*y(t0)-a1*y'(t0)=alfa,  y(tmax)=beta
%
if m(1,2)~=0 & m(2,2)==0
    A=diag((1+h/2*p(2:N)),-1)+diag((-2-h^2*q(1:N)))+diag((1-h/2*p(1:N-1)),1);
    A(1,1)=m(1,2)*A(1,1)-2*h*m(1,1)*(1+p(1)*h/2);
    A(1,2)=2*m(1,2);

    b=r(1:N)*h^2;
    b(1)=m(1,2)*b(1)-2*h*alfa*(1+p(1)*h/2);
    b(N)=b(N)-beta*(1-p(N)*h/2);
end
%
% 4 CASO:  a0*y(t0)-a1*y'(t0)=alfa,  b0*y(tmax)+b1*y'(tmax)=beta
%
if m(1,2)~=0 & m(2,2)~=0
    A=diag((1+h/2*p(2:N+1)),-1)+diag((-2-h^2*q))+diag((1-h/2*p(1:N)),1);

    A(1,1)=m(1,2)*A(1,1)-2*h*m(1,1)*(1+p(1)*h/2);
    A(1,2)=2*m(1,2);
    A(N+1,N)=2*m(2,2);
    A(N+1,N+1)=m(2,2)*A(N+1,N+1)-2*h*m(2,1)*(1-p(N+1)*h/2);

    b=r*h^2;
    b(1)=m(1,2)*b(1)-2*h*alfa*(1+p(1)*h/2);
    b(N+1)=m(2,2)*b(N+1)-2*h*beta*(1-p(N+1)*h/2);
end
if abs(det(A))<=eps
    disp('Il sistema non pu essere risolto: matrice singolare');
    Y=[];T=[];
    return
end
Y=A\b;
T=t;
if m(1,2)==0
    Y=[alfa;Y];
end
if m(2,2)==0
    Y=[Y;beta];
end
