
function [fun,Jac]=Nonlin_matdiff(t0,tmax,alfa,beta,N,f,dfi_1,dfi,dfip1);
%________________________________________________________________________
%
% Funzione Nonlin_matdiff
% ____________________
%
% Chiamata:   [fun,Jac]=Nonlin_matdiff(t0,tmax,alfa,beta,N,f,dfi_1,dfi,dfip1);
%
% Commento:
%
% La funzione 'nonlin_matdiff' costruisce i vettori di stringhe fun e Jac associate 
% ad un sistema, in generale, non lineare, ottenuto approssimando con le formule delle
% differenze finite la y' e y'' in un problema ai limiti del 2 ordine della forma:
%
% y''(t)=f(t,y(t),y'(t))     con t in (t0,tmax)
% y(t0)=alfa
% y(tmax)=beta
%
%
% dove f, alfa e beta sono assegnati.
%
%________________________________________________________________________
% 
%
% Parametri di input della function:
%
% t0,tmax    estremi dell'intervallo [t0,tmax] di integrazione;         
% alfa, beta condizioni ai limiti: y(t0)=alfa, y(tmax)=beta;
% N          numero di sottointervalli in cui e' stato suddiviso [t0,tmax]; 
% f          stringa contenente l'espressione della f del problema discretizzata:
%            f(t(i),y(i),(y(i+1)-y(i-1))/(2*h));
% dfi_1      stringa contenente l'espressione della derivata della f rispetto 
%            ad y(i-1), codiagonale inferiore della matrice Jacobiana;
% dfi        stringa contenente l'espressione della derivata della f rispetto 
%            ad y(i), diagonale della matrice Jacobiana;
% dfip1      stringa contenente l'espressione della derivata della f rispetto 
%            ad y(i+1), codiagonale superiore della matrice Jacobiana.
%
%
% Parametri di output della function:
%
%     fun        vettore colonna di dimensione N+1 in cui ogni componente 
%                e' una stringa contenente le espressioni di f1,f2,...,fN+1.
%     Jac        vettore colonna di dimensione (N+1)^2 in cui ogni componente 
%                e' una stringa contenente le espressioni delle derivate 
%                parziali delle fi, i=1,...,N+1, che costituiscono la matrice 
%                jacobiana; per n=2 Jac=[J11;J12;J21;J22] dove Jik, i,k=1,2,
%                e' la stringa della derivata di fi rispetto a yk.
%                 
%_________________________________________________________________________
%
h=(tmax-t0)/N;
t=(linspace(t0,tmax,N+1))';

% Modifica della stringa f, lasciando opportunamente spazi
k=1;m=length(f);
while k<=m
    if f(k)=='i'
        f=[f(1:k-1),'i    ',f(k+1:end)];
        m=m+4; k=k+4;
    end
    k=k+1;
end
k=1;m=length(dfi_1);
while k<=m
    if dfi_1(k)=='i'
        dfi_1=[dfi_1(1:k-1),'i    ',dfi_1(k+1:end)];
        m=m+4; k=k+4;
    end
    k=k+1;
end
k=1;m=length(dfi);
while k<=m
    if dfi(k)=='i'
        dfi=[dfi(1:k-1),'i    ',dfi(k+1:end)];
        m=m+4; k=k+4;
    end
    k=k+1;
end
k=1;m=length(dfip1);
while k<=m
    if dfip1(k)=='i'
        dfip1=[dfip1(1:k-1),'i    ',dfip1(k+1:end)];
        m=m+4; k=k+4;
    end
    k=k+1;
end


% vettore colonna contenente la funzione, da passare a Newton, di cui 
% vogliamo calcolare lo zero.
fun=[]; 
str1='y(1)-alfa';
fun=strvcat(fun,str1);
f=['-h^2*(' f ')'];
for i=2:N
    str1=['y(i    -1)-2*y(i    )+y(i    +1)' f];
    %cerco la lettera i e sostituisco il numero corrispondente
    ind=findstr(str1,'i');
    stri=num2str(i);
    for j=1:length(ind)
        for jj=1:length(stri)
            str1(ind(j)+jj-1)=stri(jj);
        end
    end
    fun=strvcat(fun,str1);
end
str1='y(N+1)-beta';
fun=strvcat(fun,str1);
% 
% costruzione del vettore Jac
%
Jac=[];
% prima riga della matrice Jacobiana
Jac=strvcat(Jac,'1');
for ii=2:N+1
    Jac=strvcat(Jac,'0');
end
% parte centrale della matrice Jacobiana
for i=2:N
    stri=num2str(i);
    %elementi extradiagonali
    for ii=1:i-2
        Jac=strvcat(Jac,'0');
    end
    %codiagonale inferiore
    str4=['1-h^2*(' dfi_1 ')'];
    ind=findstr(str4,'i');
    for j=1:length(ind)
        for jj=1:length(stri)
            str4(ind(j)+jj-1)=stri(jj);
        end
    end
    Jac=strvcat(Jac,str4);    
    %diagonale 
    str3=['-2-h^2*('  dfi ')'];
    ind=findstr(str3,'i');
    for j=1:length(ind)
        for jj=1:length(stri)
            str3(ind(j)+jj-1)=stri(jj);
        end
    end
    Jac=strvcat(Jac,str3);    
    %codiagonale superiore
    str4=['1-h^2*(' dfip1 ')'];
    ind=findstr(str4,'i');
    for j=1:length(ind)
        for jj=1:length(stri)
            str4(ind(j)+jj-1)=stri(jj);
        end
    end
    Jac=strvcat(Jac,str4);
    %elementi extradiagonali    
    for ii=N:-1:i+1
        Jac=strvcat(Jac,'0');
    end
end
% ultima riga della matrice Jacobiana
for ii=1:N
    Jac=strvcat(Jac,'0');
end
Jac=strvcat(Jac,'1');
% 
