
function [T,Y,s_ott]=Shoot_int(t0,tmax,alpha,beta,N,m,c0,c1,f,s0,smax,hs);
%________________________________________________________________________
%
% Funzione Shoot_int 
% ____________________
%
% Chiamata:   [T,Y,s_ott]=Shoot_int(t0,tmax,alpha,beta,N,m,c0,c1,f,s0,smax,hs);
%
% Commento:
%
% La funzione 'Shoot_int' risolve un problema ai limiti del secondo ordine
% utilizzando il metodo shooting. Per la determinazione del valore ottimale
% del parametro s, si fa uso dell'interpolazione polinomiale. Per
% l'integrazione del problema di Cauchy si fa uso del metodo di Runge-Kutta
% classico del quarto ordine.
% Il problema che si considera e':
%
% y''(t)=F(t,y(t),y'(t))     con t in (t0,tmax)
% a0*y(t0)-a1*y'(t0)=alfa
% b0*y(tmax)+b1*y'(tmax)=beta
%
% dove F, a0, a1, b0, b1, alfa e beta sono assegnati.
% Il metodo trasforma il problema ai limiti in uno di Cauchy del tipo:
%
% y1'(t)=y2(t)=f1(t)         con t in (t0,tmax)
% y2'(t)=F(t,y1(t),y2(t))=f2(t,y1(t),y2(t))
% y1(t0)=a1*s_ott-c1*alfa
% y2(t0)=a0*s_ott-c0*alfa
%
% I coefficienti c0 e c1 vengono definiti a priori imponendo che  c0*a1-c1*a0=1. 
% Se, in particolare, le condizioni ai limiti sono:
% y(t0)=alfa
% y(tmax)=beta
% a0=1, a1=0, allora si pu assumere c0 e c1 pari a:     c0=0 c1=-1.
%
%________________________________________________________________________
% 
%
% Parametri di input della function:
%
% t0,tmax      estremi dell'intervallo [t0,tmax] di integrazione;         
% alpha, beta  condizioni ai limiti;
% N            numero di sottointervalli in cui e' stato suddiviso [t0,tmax]; 
% m            matrice di dimensione 2 x 2 contenente i coefficienti delle 
%              combinazioni di y e y' ai limiti; si  indicato:
%
%              |a0  a1|    |m(1,1) m(1,2) |
%              |b0  b1| =  |m(2,1) m(2,2) | .
%
% c0,c1        Coefficienti che devono soddisfare la seguente relazione 
%              c0*a1-c1*a0=1;
% f            vettore colonna di dimensione 2  in cui ogni componente e'
%              una stringa contenente le espressioni f1, f2. Il vettore pu
%              essere costruito mediante il comando matlab 'f=strvcat(f1,f2)';
% s0:hs:smax   vettore (s0,s1,...,smax) (con passo s(i)-s(i-1)=hs),  dei 
%              parametri s della condizione iniziale che determinano una 
%              famiglia di problemi di Cauchy.
%        
%
% Parametri di output della function:
%
% T          vettore colonna di dimensione N+1 contenente i nodi t0,t1,...,tn;
% Y          matrice a due colonne contenente rispettivamente i valori approssimati
%            della soluzione y1 e della sua derivata y2 del problema di Cauchy, 
%            relativa al valore  ottimale del parametro s; 
% s_ott      parametro ottimale s che rende il problema di Cauchy equivalente al 
%            problema ai limiti.
%                             
% 
%_________________________________________________________________________
%
% Primo esempio  di utilizzo della function  applicata al problema ai limiti:
%
% 
% y''(t)+y(t)=t    t e' in (0,pi/2)
% -2y(0)-y'(0)=-2
% 3y(pi/2)-y'(pi/2)=3(pi/2-1)
% 
% con soluzione vera:
% y(t)=cos(t)-sin(t)+t;
% 
%
% clear all
% clc
% s0=0;hs=.25;smax=1.25;
% t0=0;tmax=pi/2;
% alpha=-2;beta=3*(pi/2-1);
% N=10;
% m=[-2 1;3 -1];
% c0=1/2; c1=1/4;
% f=strvcat('y(2)','-y(1)+t');
% 
% %Implementazione del metodo
% 
% [T,Y,s_ott]=Shoot_int(t0,tmax,alpha,beta,N,m,c0,c1,f,s0,smax,hs);
%
%
%__________________________________________________________________________
%
% Secondo esempio  di utilizzo della function  applicata al problema ai limiti:
% 
% y''(t)+y(t)=0    t e' in (0,pi/2)
% y(0)=0
% y(pi/2)=1
% 
% con soluzione vera:
% y(t)=sin(t); 
%
% clear all
% clc
% s0=-0.25;hs=0.25;smax=2;
% t0=0;tmax=pi/2;
% alpha=0;beta=1;
% N=10;
% m=[1 0; 1 0];
% c0=0; c1=-1;
% f=strvcat('y(2)','-y(1)');
%
% %Implementazione del metodo
%
%[T,Y,s_ott]=Shoot_int(t0,tmax,alpha,beta,N,m,c0,c1,f,s0,smax,hs);
%
%________________________________________________________________________
%

s=(s0:hs:smax)';
ncase=length(s);
yb=[];
for i=1:ncase
    y0=[m(1,2)*s(i)-c1*alpha,m(1,1)*s(i)-c0*alpha];%y0=[alpha,s(i)] quando y(a)=alpha, y(b)=beta
    [T,Y]=Rungekutta4(t0,tmax,N,y0,f);
    yb=[yb;Y(end,:)];
    plot(T,Y(:,1));
    hold on
end
fs=m(2,1)*yb(:,1)+m(2,2)*yb(:,2); %fs=yb(:,1) quando y(a)=alpha, y(b)=beta
p=polyfit(fs,s,ncase-1);
s_ott=polyval(p,beta);

y0=[(m(1,2)*s_ott-c1*alpha),m(1,1)*s_ott-c0*alpha];%y0=[alpha,s_ott] quando y(a)=alpha, y(b)=beta
[T,Y]=Rungekutta4(t0,tmax,N,y0,f);
plot(T,Y(:,1),'r.-');grid on;
title('Famiglia di soluzioni Y(t,si);Y(t,s ott) e'' in rosso ');
xlabel('t');ylabel('Y(t,si)');hold off




