
function [T,Y,s]=Shooting(t0,tmax,alfa,beta,N,m,c0,c1,f,s0,s1);
%________________________________________________________________________
%
% Funzione Shooting 
% ____________________
%
% Chiamata:   [T,Y,s]=Shooting(t0,tmax,alfa,beta,N,m,c0,c1,f,s0,s1);
%
% Commento:
%
% La funzione 'Shooting' utilizza il metodo delle secanti per il calcolo del 
% parametro s ed il metodo di Runge-Kutta del quarto ordine esplicito per 
% determinare in  un intervallo [t0,tmax], i valori approssimati della 
% soluzione di un problema ai limiti del 2 ordine della forma:
%
% y''(t)=F(t,y(t),y'(t))     con t in (t0,tmax)
% a0*y(t0)-a1*y'(t0)=alfa
% b0*y(tmax)+b1*y'(tmax)=beta
%
% dove F, a0, a1, b0, b1, alfa e beta sono assegnati.
% Il metodo trasforma il problema ai limiti in uno di Cauchy del tipo:
%
% y1'(t)=y2(t)=f1(t)         con t in (t0,tmax)
% y2'(t)=F(t,y1(t),y2(t))=f2(t,y1(t),y2(t))
% y1(t0)=a1*s-c1*alfa
% y2(t0)=a0*s-c0*alfa
%
% e determina il valore del parametro s che soddisfa la seguente relazione:
%             |b0*y1(tmax,s)+b1*y2(tmax,s)-beta|<=toll .
% 
% I coefficienti c0 e c1 vengono definiti a priori imponendo che  
% c0*a1-c1*a0=1.  Ad esempio, se le condizioni ai limiti sono:
% y(t0)=alfa
% y(tmax)=beta
% a0=1, a1=0, allora si pu assumere c0 e c1 pari a:     c0=0 c1=-1.
%________________________________________________________________________
% 
%
% Parametri di input della function:
%
% t0,tmax    estremi dell'intervallo [t0,tmax] di integrazione;         
% alfa, beta condizioni ai limiti;
% N          numero di sottointervalli in cui e' stato suddiviso [t0,tmax]; 
% m          matrice di dimensione 2 x 2 contenente i coefficienti delle 
%            combinazioni di y e y' ai limiti; si  indicato:
%
%            |a0  a1|    |m(1,1) m(1,2) |
%            |b0  b1| =  |m(2,1) m(2,2) | .
%
% c0,c1      Coefficienti che devono soddisfare la seguente relazione 
%            c0*a1-c1*a0=1;
% f          vettore colonna di dimensione 2  in cui ogni componente e'
%            una stringa contenente le espressioni f1, f2. Il vettore pu
%            essere costruito mediante il comando matlab 'f=strvcat(f1,f2)';
% s0,s1      valori per innescare il metodo delle secanti.
%        
%
% Parametri di output della function:
%
% T          vettore colonna di dimensione N+1 contenente i nodi t0,t1,...,tn;
% Y          matrice a due colonne contenente rispettivamente i valori approssimati
%            della soluzione y1 e della sua derivata y2 del problema di Cauchy, 
%            relativa al valore del parametro s per cui  soddisfatta la 
%            condizione di arresto;
% s          vettore colonna contenente i valori del parametro s fino a quello che 
%            soddisfa il criterio di arresto.
% 
%_________________________________________________________________________
%
% Esempio  di utilizzo della function  applicata al seguente 
% problema ai limiti:
%
% 
% y''(t)=(32+2*t^3-y(t)*y'(t))/8    t e' in (1,3)
% y(1)=17
% y(3)=43/3
% 
% con soluzione vera:
% y(t)=t^2+16/t
%
%________________________________________________________________________
%
% clear all
% clc
% s0=-1;s1=-20;
% t0=1;tmax=3;
% alfa=17;
% beta=43/3;
% m=[1 0; 1 0]; 
% c0=0; c1=-1;
% N=40;
% f=strvcat('y(2)','(32+2*t.^3-y(1).*y(2))/8');
% 
% %Implementazione del metodo
% 
% [T,Y,s]=Shooting(t0,tmax,alfa,beta,N,m,c0,c1,f,s0,s1);
% 
%________________________________________________________________________
%

toll=1.e-10;
nmax=30;
it=1;
y0=[m(1,2)*s0-c1*alfa  m(1,1)*s0-c0*alfa];           % condizioni iniziali  
[T,Ys0]=Rungekutta4(t0,tmax,N,y0,f); % calcolo della soluzione e sua derivata per s=s0
f0=m(2,1)*Ys0(N+1,1)+m(2,2)*Ys0(N+1,2)-beta;                 
df=abs(f0);
s=[s0];
if df<toll
   ('s=s0');
   Y=Ys0;
   return
end
s=[s; s1]; 
y0=[m(1,2)*s1-c1*alfa  m(1,1)*s1-c0*alfa];           % condizioni iniziali  
[T,Ys1]=Rungekutta4(t0,tmax,N,y0,f);% calcolo della soluzione e sua derivata per s=s1
f1=m(2,1)*Ys1(N+1,1)+m(2,2)*Ys1(N+1,2)-beta;                 

df=abs(f1);
if df<toll
   ('s=s1');
   Y=Ys1;
   return
end
while it<nmax & df>toll & abs(f1-f0)>eps  
   s2=s1-f1*(s1-s0)/(f1-f0);   % metodo delle secanti
   y0=[m(1,2)*s2-c1*alfa  m(1,1)*s2-c0*alfa];           % condizioni iniziali  
   [T,Y]=Rungekutta4(t0,tmax,N,y0,f);  % calcolo della soluzione e sua derivata per s=s2
   f2=m(2,1)*Y(N+1,1)+m(2,2)*Y(N+1,2)-beta;                 
   df=abs(f2);
   it=it+1;
   s0=s1; s1=s2;
   f0=f1; f1=f2;
   s=[s; s2];
end
